/**
 * apeman bud for React files.
 * @memberof module:apeman-bud-react/lib
 * @function setup
 * @param {object} params - Template params.
 * @returns {object} - Coz bud configuration.
 */

"use strict";

const argx = require('argx'),
    assert = require('assert'),
    path = require('path'),
    stringcase = require('stringcase'),
    expandglob = require('expandglob'),
    _bud = require('./_bud');


/** @lends setup */
function setup(params) {
    let args = argx(arguments);
    params = args.shift('object') || {};

    let type = params.type,
        name = params.name;

    assert(name, "params.name is required.");

    return [].concat(name).map(name =>
        _bud(type, name, params)
    );
}

setup.view = function (name, params) {
    return [].concat(name).map(name =>
        _bud('view', name, Object.assign({
            suffix: 'view'
        }, params))
    );
};

setup.page = function (name, params) {
    return [].concat(name).map(name =>
        _bud('page', name, Object.assign({
            suffix: 'page'
        }, params))
    ).concat(
        _bud('page-base', 'page', {filename: 'page.jsx'})
    )
};

setup.mixin = function (name, params) {
    return [].concat(name).map(name =>
        _bud('mixin', name, Object.assign({
            suffix: 'mixin'
        }, params))
    )
};

setup.mixinGroup = function (name, params) {
    return [].concat(name).map(name =>
        _bud('mixin-group', name, Object.assign({
            suffix: 'mixin_group'
        }, params))
    )
};

setup.service = function (name, params) {
    return [].concat(name).map(name =>
        _bud('service', name, Object.assign({
            suffix: 'service'
        }, params))
    ).concat(
        _bud('service-base', 'service', {filename: 'service.jsx'})
    )
};

setup.store = function (name, params) {
    return [].concat(name).map(name =>
        _bud('store', name, Object.assign({
            suffix: 'store'
        }, params))
    )
};

setup.storeAccess = function (name, params) {
    return [].concat(name).map(name =>
        _bud('storeAccess', name, Object.assign({
            suffix: 'access'
        }, params))
    )
};

setup.storeReduce = function (name, params) {
    return [].concat(name).map(name =>
        _bud('storeReduce', name, Object.assign({
            suffix: 'reduce'
        }, params))
    )
};

setup.fragment = function (name, params) {
    return [].concat(name).map(name =>
        _bud('fragment', name, Object.assign({
            suffix: ''
        }, params))
    )
};

setup.browser = function (name, params) {
    return [].concat(name).map(name =>
        _bud('browser', name, Object.assign({
            extname: '.js'
        }, params))
    )
};

setup.helper = function (name, params) {
    return [].concat(name).map(name =>
        _bud('helper', name, Object.assign({
            extname: '.jsx',
            suffix: 'helper'
        }, params))
    )
};

setup.index = function (dirname, params) {
    params = params || {};


    let bud = _bud('index', 'index', Object.assign({
        suffix: '',
        extname: '.jsx',
        modules: (dirname =>
            expandglob.sync('**/+(*.js|*.jsx)', {cwd: dirname})
                .map(filename => (filename.replace(/\.js(x?)$/, '')))
                .filter(filename => (!/^index$/.test(path.basename(filename))))
                .filter(filename => (!/^_/.test(path.basename(filename))))
                .filter((filename, i, array) => (array.indexOf(filename) === i))
                .map(filename => ({
                    filename: filename,
                    get name() {
                        let name = path.basename(filename);
                        return params.camelcase ? stringcase.camelcase(name) : stringcase.pascalcase(name)
                    }
                })))(dirname || params.dirname)
    }, params));
    bud.force = true;
    return bud;
};


module.exports = setup;
