/**
 * Test case for setup.
 * Runs with mocha.
 */
"use strict";

const setup = require('../lib/setup.js'),
    assert = require('assert'),
    coz = require('coz');

describe('setup', () => {


    before((done) => {
        done();
    });

    after((done) => {
        done();
    });


    it('Render views', (done) => {
        let bud = setup({
            name: [
                'SplashTop',
                'AccountDetail'
            ],
            type: 'view'
        });
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/views/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render fragments', (done) => {
        let bud = setup({
            name: [
                'foo/SampleFragment',
                'bar/sample_baz_baz'
            ],
            type: 'fragment'
        });
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/fragments/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render pages', (done) => {
        let bud = setup.page([
            'sample_page_bar'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/pages/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render mixins', (done) => {
        let bud = setup({
            name: [
                'bar/sample_mixin_bar'
            ],
            type: 'mixin'
        });
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/mixins/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render mixin groups', (done) => {
        let bud = setup.mixinGroup([
            'bar/Viewable'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/mixins/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render services', (done) => {
        let bud = setup.service([
            'bar/sample_service_bar'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/services/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render stores', (done) => {
        let bud = setup.store([
            'bar/sample_store_bar'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/stores/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render store accesses', (done) => {
        let bud = setup.storeAccess([
            'dispatch'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/stores/accessing/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render store reduces', (done) => {
        let bud = setup.storeReduce([
            'listing'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/stores/reducing/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render themes', (done) => {
        let bud = setup({
            name: [
                'foo/SampleTheme',
                'bar/sample_themes'
            ],
            type: 'fragment'
        });
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/themes/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render browser', (done) => {
        let bud = setup.browser([
            'Index'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/browser/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render browser render page', (done) => {
        let bud = setup.browser([
            'render-page'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/browser/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render fragments index', (done) => {
        let bud = setup.index(__dirname + '/../lib');
        assert.ok(bud);
        assert.ok(bud.tmpl);
        bud.path = __dirname + '/../tmp/fragments/' + bud.path;
        bud.mkdirp = true;
        bud.force = true;
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render pre defined fragments', (done) => {

        let bud = setup.fragment([
            'footer',
            'header',
            'main',
            'theme'
        ]);
        bud.forEach((bud) => {
            assert.ok(bud);
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/fragments/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render views index', (done) => {
        let bud = setup.index(__dirname + '/../lib');
        assert.ok(bud);
        assert.ok(bud.tmpl);
        bud.path = __dirname + '/../tmp/views/' + bud.path;
        bud.mkdirp = true;
        bud.force = true;
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render theme', (done) => {
        let bud = setup.fragment('Theme');
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/theme/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Render helper', (done) => {
        let bud = setup.helper([
            'foo'
        ]);
        assert.ok(bud);
        bud.forEach((bud) => {
            assert.ok(bud.tmpl);
            bud.path = __dirname + '/../tmp/helpers/' + bud.path;
            bud.mkdirp = true;
            bud.force = true;
        });
        coz.render(bud, (err) => {
            assert.ifError(err);
            done();
        });
    });


});

